/**
 * Define rview server endpoints
 * @function endpoints
 * @returns {RView} -  A rview instance
 */
'use strict'

const { spinalcase } = require('stringcase')
const rview = require('rview')
const proxy = require('apeman-middleware-proxy')
const sse = require('apeman-middleware-sse')
const SCOPE_PREFIX = /^@/
const SETTING_PREFIX = /^\$/

/** @lends endpoints */
function endpoints (modules) {
  let {
    base,
    scoped,
    settings
  } = Object.keys(modules).reduce((formed, key) => {
    let value = modules[ key ]
    if (SETTING_PREFIX.test(key)) {
      formed.settings[ key ] = value
    } else if (SCOPE_PREFIX.test(key)) {
      formed.scoped[ key ] = value
    } else {
      formed.base[ key ] = value
    }
    return formed
  }, { base: {}, scoped: {}, settings: {} })
  let { $serverMiddlewares, $proxy, $sse } = settings
  delete settings.$serverMiddlewares
  delete settings.$proxy
  delete settings.$sse
  $serverMiddlewares = [].concat(
    ...Object.keys($proxy || {}).map((pattern) => proxy(pattern, $proxy[ pattern ])),
    ...Object.keys($sse || {}).map((pattern) => sse(pattern, $sse[ pattern ])),
    $serverMiddlewares || []
  )
  return rview([
    (Object.assign({}, settings, { $serverMiddlewares }, base)),
    ...Object.keys(scoped).map((name) => {
      let $pathname = `${settings.$pathname}/${spinalcase(name.replace(SCOPE_PREFIX, '')).trim()}`
      return Object.assign({}, settings, { $pathname }, scoped[ name ])
    })
  ])
}

module.exports = endpoints
