/**
 * Run ui server
 * @function uiServer
 * @param {Object} $ui
 * @param {Object} [options={}] - Optional settings
 * @returns {Promise}
 */
'use strict'

const co = require('co')
const http = require('http')
const endpoints = require('./endpoints')

/** @lends uiServer */
function uiServer ($ui, options = {}) {
  return co(function * () {
    if (!$ui) {
      throw new Error('$ui is required')
    }
    $ui = Object.assign({
      $pathname: ''
    }, $ui)
    let server = http.createServer()

    endpoints($ui).attachTo(server)

    let { listen, close } = server
    return Object.assign(server, {
      $pathname: $ui.$pathname,
      // Wrap listen to promise
      listen (port) {
        return new Promise((resolve, reject) =>
          listen.call(server, port, (err) => err ? reject(err) : resolve())
        )
      },
      // Wrap close to promise
      close () {
        return new Promise((resolve, reject) =>
          close.call(server, (err) => err ? reject(err) : resolve())
        )
      }
    })
  })
}

Object.assign(uiServer, {})

module.exports = uiServer
