/**
 * Test case for apemanUi.
 * Runs with mocha.
 */
'use strict'

const apemanUi = require('../lib/apeman_ui.js')
const arequest = require('arequest')
const aport = require('aport')
const assert = require('assert')
const EventSource = require('eventsource')
const asleep = require('asleep')
const co = require('co')

describe('apeman-ui', function () {
  let port, server
  let request = arequest.create({})
  before(() => co(function * () {
    port = yield aport()
    server = yield apemanUi({
      port,
      configuration: {
        $ui: {
          // Define Views
          'user': {
            profile: co.wrap(function * userProfile () {
              return `
<!DOCTYPE html><html><!-- ... --></html>
`
            })
          },
          $compile (content) {
            return `<!DOCTYPE html>${content}`
          },
          $proxy: {
            '/api/*': 'http://localhost:3001'
          },
          $sse: {
            '/sse/bomb': (client, ctx) => co(function * () {
              let count = 3
              while (count >= 0) {
                client.send('tick', JSON.stringify(count))
                yield asleep(100)
                count--
              }
              client.send('boom!')
            })
          }
        }
      }
    })
  }))

  after(() => co(function * () {
    server.close()
  }))

  it('Command', () => co(function * () {
    let { statusCode, body } = yield request({
      url: `http://localhost:${port}/user/profile`,
      method: 'GET'
    })
    assert.equal(statusCode, 200)
    assert.equal(body, '<!DOCTYPE html>\n<!DOCTYPE html><html><!-- ... --></html>\n')

    yield asleep(100)
  }))

  it('Event source', () => co(function * () {
    let eventSource = new EventSource(`http://localhost:${port}/sse/bomb`)
    eventSource.on('tick', (event) => {
      let { data } = event
      console.log('tick', data)
    })

    yield asleep(1500)
  }))
})

/* global describe, before, after, it */
